@@ -456,7 +456,22 @@ sub _compress_upx ( $self, $path ) {
         if (@files) {
             say q[];
 
-            P->pm->run_check( $upx, '--best', @files ) or 1;
+            my $cmd = q[];
+
+            for my $file (@files) {
+                if ( length qq[$cmd "$file"] > 8191 ) {
+                    P->pm->run_check($cmd) or 1;
+
+                    $cmd = qq[$upx --best "$file"];
+                }
+                else {
+                    $cmd ||= qq[$upx --best];
+
+                    $cmd .= qq[ "$file"];
+                }
+            }
+
+            P->pm->run_check($cmd) or 1 if $cmd;
 
             for my $file (@files) {
                 P->file->copy( $file, $upx_cache_dir . $file_md5->{$file} );
@@ -612,19 +627,19 @@ sub _error ( $self, $msg ) {
 ## │ Sev. │ Lines                │ Policy                                                                                                         │
 ## ╞══════╪══════════════════════╪════════════════════════════════════════════════════════════════════════════════════════════════════════════════╡
 ## │    3 │ 174, 193, 200, 232,  │ References::ProhibitDoubleSigils - Double-sigil dereference                                                    │
-## │      │ 369, 410, 547        │                                                                                                                │
+## │      │ 369, 410, 562        │                                                                                                                │
 ## ├──────┼──────────────────────┼────────────────────────────────────────────────────────────────────────────────────────────────────────────────┤
 ## │    3 │ 246                  │ Subroutines::ProhibitManyArgs - Too many arguments                                                             │
 ## ├──────┼──────────────────────┼────────────────────────────────────────────────────────────────────────────────────────────────────────────────┤
-## │    3 │ 444                  │ ValuesAndExpressions::ProhibitMismatchedOperators - Mismatched operator                                        │
+## │    3 │ 444, 462             │ ValuesAndExpressions::ProhibitMismatchedOperators - Mismatched operator                                        │
 ## ├──────┼──────────────────────┼────────────────────────────────────────────────────────────────────────────────────────────────────────────────┤
-## │    3 │ 482                  │ RegularExpressions::ProhibitCaptureWithoutTest - Capture variable used outside conditional                     │
+## │    3 │ 497                  │ RegularExpressions::ProhibitCaptureWithoutTest - Capture variable used outside conditional                     │
 ## ├──────┼──────────────────────┼────────────────────────────────────────────────────────────────────────────────────────────────────────────────┤
-## │    2 │ 517                  │ ValuesAndExpressions::ProhibitLongChainsOfMethodCalls - Found method-call chain of length 4                    │
+## │    2 │ 532                  │ ValuesAndExpressions::ProhibitLongChainsOfMethodCalls - Found method-call chain of length 4                    │
 ## ├──────┼──────────────────────┼────────────────────────────────────────────────────────────────────────────────────────────────────────────────┤
-## │    2 │ 569, 571             │ ValuesAndExpressions::ProhibitEscapedCharacters - Numeric escapes in interpolated string                       │
+## │    2 │ 584, 586             │ ValuesAndExpressions::ProhibitEscapedCharacters - Numeric escapes in interpolated string                       │
 ## ├──────┼──────────────────────┼────────────────────────────────────────────────────────────────────────────────────────────────────────────────┤
-## │    1 │ 504, 510             │ CodeLayout::ProhibitParensWithBuiltins - Builtin function called with parentheses                              │
+## │    1 │ 519, 525             │ CodeLayout::ProhibitParensWithBuiltins - Builtin function called with parentheses                              │
 ## └──────┴──────────────────────┴────────────────────────────────────────────────────────────────────────────────────────────────────────────────┘
 ##
 ## -----SOURCE FILTER LOG END-----
